//----------------------------------------------------------------------------------------------------//

/* 
 *	Jan Koci Software
 *	http://koci.opengl.cz
 */

//----------------------------------------------------------------------------------------------------//

/* 
 * KRegistryKey.cpp
 * 
 * Source file for registry key application
 * This application read and write to the windows registry key.
 * Read windows program files path from registry
 * Write application path to the registry
 * Read application path from registry
 * Read and write some double value to the registry
 * 
 *
 */

//----------------------------------------------------------------------------------------------------//

/*
 * 05.01.21 - First version
 *			
 */

//----------------------------------------------------------------------------------------------------//

// KRegistryKey.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
using namespace std;


//----------------------------------------------------------------------------------------------------//

#define K_REGISTRY_KEY_NAME "Software\\JanKoci\\Tutorial"
#define K_REGISTRY_KEY_WIN "SOFTWARE\\Microsoft\\Windows\\CurrentVersion"
#define K_REGISTRY_ROOT_PATH "RootPath"
#define K_REGISTRY_INT "Int"
#define K_REGISTRY_BOOL "Bool"


//----------------------------------------------------------------------------------------------------//

// Init application
void CRegistryKey::Init()
{
	// Get application current path
	GetCurrentDirectory(MAX_PATH, m_strCurrPath);
	// Set Read and write values
	m_i = 12345;
	m_b = true;	
}

//----------------------------------------------------------------------------------------------------//

// Open registy key
HKEY CRegistryKey::RegistryOpenKey(HKEY hKey, LPCTSTR lpKeyName, LPBOOL lpbCreated)
{
	DWORD dwDisp = 0;
	HKEY hkResult = NULL;
	if ( RegCreateKeyEx(hKey, lpKeyName, 0, NULL, REG_OPTION_NON_VOLATILE,
		KEY_ALL_ACCESS, NULL, &hkResult, &dwDisp) != ERROR_SUCCESS )
		return NULL;
	if ( lpbCreated )
		*lpbCreated = ( dwDisp & REG_CREATED_NEW_KEY );
	return hkResult;
}

//----------------------------------------------------------------------------------------------------//

// Close registry key
BOOL CRegistryKey::RegistryCloseKey(HKEY hKey)
{
	return ( RegCloseKey(hKey) == ERROR_SUCCESS);
}

//----------------------------------------------------------------------------------------------------//

// Write to the registry word value
BOOL CRegistryKey::RegistryWriteDWORD(HKEY hKey, LPCTSTR lpValueName, DWORD dwValue)
{
	return  ( RegSetValueEx(hKey, lpValueName, 0, REG_DWORD,
		(CONST BYTE*)&dwValue, sizeof(DWORD)) == ERROR_SUCCESS );
}

//----------------------------------------------------------------------------------------------------//

// Write to the registry string value
BOOL CRegistryKey::RegistryWriteString(HKEY hKey, LPCTSTR lpValueName, LPCTSTR lpText)
{
	return ( RegSetValueEx(hKey, lpValueName, 0, REG_SZ, (CONST BYTE*)lpText,
		(lstrlen(lpText)+1)*sizeof(TCHAR)) ==  ERROR_SUCCESS );
}

//----------------------------------------------------------------------------------------------------//

// Read from registry word value
BOOL CRegistryKey::RegistryReadDWORD(HKEY hKey, LPCTSTR lpValueName, LPDWORD lpdwResult)
{
  DWORD dwValue, dwSize, dwType;
  dwSize = sizeof(DWORD);
  dwValue = 0;
  if ( RegQueryValueEx(hKey, lpValueName, NULL, &dwType, (LPBYTE)&dwValue, &dwSize)
    != ERROR_SUCCESS )
    return FALSE;
  *lpdwResult = dwValue;
  return TRUE;
}


//----------------------------------------------------------------------------------------------------//

// Read from registry string value
BOOL CRegistryKey::RegistryReadString(HKEY hKey, LPCTSTR lpValueName, LPTSTR lpString)
{
	DWORD dwSize, dwType;
	dwSize = 256;
	if ( RegQueryValueEx(hKey, lpValueName, NULL, &dwType, (LPBYTE)lpString, &dwSize)
		!= ERROR_SUCCESS )
		return FALSE;
	return TRUE;
}

//----------------------------------------------------------------------------------------------------//

// Get windows program files path
bool CRegistryKey::GetWinProgramFilesPath(char *strPath)
{
	// Open registry key
	HKEY hKey = RegistryOpenKey(HKEY_LOCAL_MACHINE, K_REGISTRY_KEY_WIN, NULL);
	if(hKey)
	{
		// Read from registry key
		RegistryReadString(hKey, "ProgramFilesDir", strPath);
		// Close registry key
		RegistryCloseKey(hKey);
	}
	else
	{
		CDebug::InfoMsg("CRegistryKey::GetWinProgramFilesPath() -- Don`t load win program files from registry");
		return false;
	}

	return true;
}

//----------------------------------------------------------------------------------------------------//

// Save values to the registry
void CRegistryKey::Save()
{
	// Open registry key
	HKEY hKey = RegistryOpenKey(HKEY_CURRENT_USER, K_REGISTRY_KEY_NAME, NULL);
	if(hKey)
	{
		/////////////////////////////
		// Write to the registry key
		////////////////////////
		
		// Write application path
		if(!RegistryWriteString(hKey, K_REGISTRY_ROOT_PATH, m_strCurrPath))
		{
			CDebug::InfoMsg("CRegistryKey::Save() -- Don`t write root path to the registry key");
		}
		// Write int value
		if(!RegistryWriteDWORD(hKey, K_REGISTRY_INT, m_i))
		{
			CDebug::InfoMsg("CRegistryKey::Save() -- Don`t write int value to the registry key");
		}
		// Write bool value
		if(!RegistryWriteDWORD(hKey, K_REGISTRY_BOOL, m_b))
		{
			CDebug::InfoMsg("CRegistryKey::Save() -- Don`t write bool value to the registry key");
		}
		
		// Close registry key
		if(!RegistryCloseKey(hKey))
		{
			CDebug::InfoMsg("CRegistryKey::Save() -- Don`t close registry key");
		}
	}
	else
	{
		CDebug::InfoMsg("CRegistryKey::Save() -- Don`t open registry key");
	}

}

//----------------------------------------------------------------------------------------------------//

// Load values from registry
void CRegistryKey::Load()
{
	// Open registry key
	HKEY hKey = RegistryOpenKey(HKEY_CURRENT_USER, K_REGISTRY_KEY_NAME, NULL);
	if(hKey)
	{
		/////////////////////////////
		// Read from registry key
		////////////////////////

		// Read application path
		if(!RegistryReadString(hKey, K_REGISTRY_ROOT_PATH, m_strCurrPath))
		{
			CDebug::InfoMsg("CRegistryKey::Load() -- Don`t read root path from registry key");
		}
		
		// Read int value
		DWORD dwInt;
		if(!RegistryReadDWORD(hKey, K_REGISTRY_INT, &dwInt))
		{
			CDebug::InfoMsg("CRegistryKey::Load() -- Don`t read int value from registry key");
		}
		else
		{
			// Get int from dword
			m_i = (int) dwInt;
		}

		// Read bool value
		DWORD dwBool;
		if(!RegistryReadDWORD(hKey, K_REGISTRY_BOOL, &dwBool))
		{
			CDebug::InfoMsg("CRegistryKey::Load() -- Don`t read bool value from registry key");
		}
		else
		{
			// Get bool from dword
			m_b = (bool) dwBool;
		}

		// Close registry key
		if(!RegistryCloseKey(hKey))
		{
			CDebug::InfoMsg("CRegistryKey::Load() -- Don`t close registry key");
		}
	}
	else
	{
		CDebug::InfoMsg("CRegistryKey::Load() -- Don`t open registry key");
	}
}

//----------------------------------------------------------------------------------------------------//

// Start application
void CRegistryKey::Start()
{
	cout << "Read windows program files path from registry key" << endl ;
	// Set program files path
	char strProgramFilesPath[256];
	// Get windows program files path
	GetWinProgramFilesPath(strProgramFilesPath);
	cout << "Program files path is " << strProgramFilesPath << endl ;
	cout << "" << endl ;
	cout << "Get actual application path, int and bool values" << endl ;
	//Init file paths
	Init();
	cout << "Application path is " <<  m_strCurrPath << endl ;
	cout << "Int value is " <<  m_i << endl ;
	cout << "Bool value is " <<  m_b << endl ;
	cout << "" << endl ;
	cout << "Write application path to the registry key" << endl ;
	Save();
	// Erase current path
	memset(m_strCurrPath, 0, 256);
	// Reset values
	m_i = 0;
	m_b = false;
	cout << "Erase application path"  << m_strCurrPath << endl;
	cout << "Reset int and bool values " << m_i << " " << m_b << endl;
	cout << "" << endl ;
	cout << "Read application path, int and bool values from registry key" << endl;
	Load();
	cout << "Application path is " << m_strCurrPath << endl ;
	cout << "Int value is " <<  m_i << endl ;
	cout << "Bool value is " <<  m_b << endl ;
	cout << "" << endl ;
}

//----------------------------------------------------------------------------------------------------//


// Main function
int main(int argc, char* argv[])
//int _tmain(int argc, _TCHAR* argv[])
{
	CRegistryKey kRegistryKey;
	// Start application
	kRegistryKey.Start();

	system("pause");

	return 0;
}

//----------------------------------------------------------------------------------------------------//






